#!/usr/bin/env python3

import sys
import os
import time
import argparse
from biobb_common.configuration import settings
from biobb_common.tools import file_utils as fu
from biobb_io.api.pdb import Pdb
from biobb_model.model.fix_side_chain import FixSideChain
from biobb_model.model.mutate import Mutate
from biobb_md.gromacs.pdb2gmx import Pdb2gmx
from biobb_md.gromacs.editconf import Editconf
from biobb_md.gromacs.solvate import Solvate
from biobb_md.gromacs.grompp import Grompp
from biobb_md.gromacs.genion import Genion
from biobb_md.gromacs.mdrun import Mdrun
from biobb_analysis.gromacs.gmx_rms import GMXRms
from biobb_analysis.gromacs.gmx_rgyr import GMXRgyr
from biobb_analysis.gromacs.gmx_energy import GMXEnergy
from biobb_analysis.gromacs.gmx_image import GMXImage
from biobb_analysis.gromacs.gmx_trjconv_str import GMXTrjConvStr

# Loading the biobb configuration reader
conf = settings.ConfReader(sys.argv[1])

global_log, _ = fu.get_logs(path=conf.get_working_dir_path())
global_prop = conf.get_prop_dic(global_log=global_log)
global_paths = conf.get_paths_dic()

global_log.info("step1_pdb: Download the initial Structure")
Pdb(**global_paths["step1_pdb"], properties=global_prop["step1_pdb"]).launch()

global_log.info("step2_fixsidechain: Modeling the missing heavy atoms in the structure side chains")
FixSideChain(**global_paths["step2_fixsidechain"], properties=global_prop["step2_fixsidechain"]).launch()

for mutation in conf.properties['mutations_list']:

    mut_paths = conf.get_paths_dic(mutation)
    mut_prop = conf.get_prop_dic(mutation, global_log=global_log)

    mut_paths['step3_mutate']['input_pdb_path'] = global_paths['step2_fixsidechain']['output_pdb_path']

    global_log.info("step3_mutate: Modeling a particular residue mutation")
    Mutate(**mut_paths["step3_mutate"], properties=mut_prop["step3_mutate"]).launch()

    global_log.info("step4_pdb2gmx: Generate the topology")
    Pdb2gmx(**mut_paths["step4_pdb2gmx"], properties=mut_prop["step4_pdb2gmx"]).launch()

    global_log.info("step5_editconf: Create the solvent box")
    Editconf(**mut_paths["step5_editconf"], properties=mut_prop["step5_editconf"]).launch()

    global_log.info("step6_solvate: Fill the solvent box with water molecules")
    Solvate(**mut_paths["step6_solvate"], properties=mut_prop["step6_solvate"]).launch()

    global_log.info("step7_grompp_genion: Preprocess ion generation")
    Grompp(**mut_paths["step7_grompp_genion"], properties=mut_prop["step7_grompp_genion"]).launch()

    global_log.info("step8_genion: Ion generation")
    Genion(**mut_paths["step8_genion"], properties=mut_prop["step8_genion"]).launch()

    global_log.info("step9_grompp_min: Preprocess energy minimization")
    Grompp(**mut_paths["step9_grompp_min"], properties=mut_prop["step9_grompp_min"]).launch()

    global_log.info("step10_mdrun_min: Execute energy minimization")
    Mdrun(**mut_paths["step10_mdrun_min"], properties=mut_prop["step10_mdrun_min"]).launch()

    global_log.info("step11_energy_min: Compute potential energy during minimization")
    GMXEnergy(**mut_paths["step11_energy_min"], properties=mut_prop["step11_energy_min"]).launch()

    global_log.info("step12_grompp_nvt: Preprocess system temperature equilibration")
    Grompp(**mut_paths["step12_grompp_nvt"], properties=mut_prop["step12_grompp_nvt"]).launch()

    global_log.info("step13_mdrun_nvt: Execute system temperature equilibration")
    Mdrun(**mut_paths["step13_mdrun_nvt"], properties=mut_prop["step13_mdrun_nvt"]).launch()

    global_log.info("step14_energy_nvt: Compute temperature during NVT equilibration")
    GMXEnergy(**mut_paths["step14_energy_nvt"], properties=mut_prop["step14_energy_nvt"]).launch()

    global_log.info("step15_grompp_npt: Preprocess system pressure equilibration")
    Grompp(**mut_paths["step15_grompp_npt"], properties=mut_prop["step15_grompp_npt"]).launch()

    global_log.info("step16_mdrun_npt: Execute system pressure equilibration")
    Mdrun(**mut_paths["step16_mdrun_npt"], properties=mut_prop["step16_mdrun_npt"]).launch()

    global_log.info("step17_energy_npt: Compute Density & Pressure during NPT equilibration")
    GMXEnergy(**mut_paths["step17_energy_npt"], properties=mut_prop["step17_energy_npt"]).launch()

    global_log.info("step18_grompp_md: Preprocess free dynamics")
    Grompp(**mut_paths["step18_grompp_md"], properties=mut_prop["step18_grompp_md"]).launch()

    global_log.info("step19_mdrun_md: Execute free molecular dynamics simulation")
    Mdrun(**mut_paths["step19_mdrun_md"], properties=mut_prop["step19_mdrun_md"]).launch()

    global_log.info("step20_rmsfirst: Compute Root Mean Square deviation against equilibrated structure (first)")
    GMXRms(**mut_paths["step20_rmsfirst"], properties=mut_prop["step20_rmsfirst"]).launch()

    global_log.info("step21_rmsexp: Compute Root Mean Square deviation against minimized structure (exp)")
    GMXRms(**mut_paths["step21_rmsexp"], properties=mut_prop["step21_rmsexp"]).launch()

    global_log.info("step22_rgyr: Compute Radius of Gyration to measure the protein compactness during the free MD simulation")
    GMXRgyr(**mut_paths["step22_rgyr"], properties=mut_prop["step22_rgyr"]).launch()

    global_log.info("step23_image: Imaging the resulting trajectory")
    GMXImage(**mut_paths["step23_image"], properties=mut_prop["step23_image"]).launch()

    global_log.info("step24_dry: Removing water molecules and ions from the resulting structure")
    GMXTrjConvStr(**mut_paths["step24_dry"], properties=mut_prop["step24_dry"]).launch()
